﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using KeysightSD1;

namespace GenExtTrigger_M320xA
{
    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("**** External Trigger Generation using M3xxxA PXI Module ****");
            Console.WriteLine("Please insert chassis and slot number of the M3xxxA PXI Module to be used as an ext. trigger generator.");
            Console.WriteLine("This executable allows to use only M320xA AWG Modules as external trigger generators.");
            //Console.WriteLine("Please insert: 1->to use an M320xA AWG, 2->to use an M310xA Digitizer");
            int moduleType = 1; //Convert.ToInt32(Console.ReadLine());
            while ((moduleType != 1) && (moduleType != 2))
            {
                Console.WriteLine("Unexpected value! 1->to use an M320xA AWG, 2->to use an M310xA Digitizer");
                moduleType = Convert.ToInt32(Console.ReadLine());
            }
            Console.WriteLine("Insert the Chassis Number:");
            int chassisNumber = Convert.ToInt32(Console.ReadLine());
            Console.WriteLine("Insert the Slot Number:");
            int slotNumber = Convert.ToInt32(Console.ReadLine());

            SD_AOU extTrigModule = openModule(chassisNumber, slotNumber);

            //Initialize FP trigger port
            if (moduleType == 1)
            {
                var aou = ((SD_AOU)extTrigModule);
                aou.triggerIOdirection(KeysightSD1.SD_TriggerDirections.AOU_TRG_OUT);
                aou.triggerIOwrite(0);
            }
            else if (moduleType == 2)
            {
                //var ain = ((SD_AIN)extTrigModule);
                //ain.triggerIOdirection(KeysightSD1.SD_TriggerDirections.AOU_TRG_OUT);
                //ain.triggerIOwrite(0);
            }

            //FP trigger loop
            string controlKey = "0";
            while (controlKey != "q")
            {
                Console.WriteLine("Press a key to send a FP trigger, press q to exit:");
                controlKey = Console.ReadLine();

                if (controlKey == "q")
                    return;
                else
                    triggerFP(extTrigModule, moduleType);
            }
        }

        public static void triggerFP(SD_Module module, int moduleType)
        {
            //Sends a pulse out of the FP TRIG Port
            if (moduleType == 1)
            {
                var aou = ((SD_AOU)module);
                aou.triggerIOwrite(0);
                aou.triggerIOwrite(1);
                aou.triggerIOwrite(0);
            }
            else if (moduleType == 2)
            {
                var ain = ((SD_AIN)module);
                ain.triggerIOwrite(0);
                ain.triggerIOwrite(1);
                ain.triggerIOwrite(0);
            }
        }
        static SD_AOU openModule(int chassisNumber, int slotNumber)
        {
            SD_AOU extTrigModule = new SD_AOU(); //Create an instance of the AWG module
            string partNumber = ""; //Allows to open any AWG model to use it as an external trigger generator
            int status;

            //Open a PXI module to serve as extTrigger controller
            if ((status = extTrigModule.open(partNumber, chassisNumber, slotNumber)) < 0)
            {
                Console.WriteLine("PXI module Name, Chassis or Slot numbers might be invalid!");
                Console.WriteLine("Error code:" + status.ToString() + " - " + KeysightSD1.SD_Error.getErrorMessage(status));
                Console.ReadKey();
                throw new System.Exception();
            }

            //Returns module object
            return extTrigModule;
        }
    }
}
